//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Base/Pixel/Pixel.h
//! @brief     Defines and implements interface Pixel.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif // SWIG
#ifndef BORNAGAIN_BASE_PIXEL_PIXEL_H
#define BORNAGAIN_BASE_PIXEL_PIXEL_H

#include <heinz/Vectors3D.h>

class Bin1D;

//! A rectangular pixel in a two-dimensional SphericalDetector.

class Pixel {
public:
    Pixel(const Bin1D& alpha_bin, const Bin1D& phi_bin);

    Pixel* clone() const;
    Pixel* createZeroSizePixel(double x, double y) const;
    R3 getK(double x, double y, double wavelength) const;
    double integrationFactor(double x, double y) const;
    double solidAngle() const;

private:
    const double m_alpha;
    const double m_phi;
    const double m_dalpha;
    const double m_dphi;
    mutable double m_solid_angle;
};

#endif // BORNAGAIN_BASE_PIXEL_PIXEL_H
