//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Mask/MaskViewFactory.cpp
//! @brief     Implements class MaskViewFactory
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Mask/MaskViewFactory.h"
#include "GUI/Model/Data/ProjectionItems.h"
#include "GUI/Model/Mask/MaskItems.h"
#include "GUI/View/Mask/AllMasksDisplay.h"
#include "GUI/View/Mask/RegionOfInterestDisplay.h"
#include "GUI/View/Shape/EllipseDisplay.h"
#include "GUI/View/Shape/LineViews.h"
#include "GUI/View/Shape/MaskContainerDisplay.h"
#include "GUI/View/Shape/PolygonDisplay.h"
#include "GUI/View/Shape/PolygonPointDisplay.h"

#include <boost/polymorphic_cast.hpp>

using boost::polymorphic_downcast;

IMaskElementDisplay* MaskViewFactory::createMaskView(MaskItemObject* item, SceneAdaptor* adaptor)
{
    IMaskElementDisplay* result(nullptr);

    if (auto* mask = dynamic_cast<MaskContainerItem*>(item))
        result = new MaskContainerDisplay(mask);

    else if (auto* mask = dynamic_cast<ProjectionContainerItem*>(item))
        result = new MaskContainerDisplay(mask);

    else if (auto* mask = dynamic_cast<RectangleItem*>(item))
        result = new RectangleDisplay(mask);

    else if (auto* mask = dynamic_cast<PolygonItem*>(item))
        result = new PolygonDisplay(mask);

    else if (auto* mask = dynamic_cast<PolygonPointItem*>(item))
        result = new PolygonPointDisplay(mask);

    else if (auto* mask = dynamic_cast<VerticalLineItem*>(item))
        result = new VerticalLineView(mask);

    else if (auto* mask = dynamic_cast<HorizontalLineItem*>(item))
        result = new HorizontalLineView(mask);

    else if (auto* mask = dynamic_cast<EllipseItem*>(item))
        result = new EllipseDisplay(mask);

    else if (auto* mask = dynamic_cast<MaskAllItem*>(item))
        result = new AllMasksDisplay(mask);

    else if (auto* mask = dynamic_cast<RegionOfInterestItem*>(item))
        result = new RegionOfInterestDisplay(mask);

    else
        ASSERT_NEVER;

    result->setViewSceneAdaptor(adaptor);

    return result;
}
